#include "common.h"

#include "cfg.h"
#include "plugin.h"

#include <libini.h>

#define	INIT_CFG_DATA()		memset(&data,0,sizeof(ini_data));	\
							memset(list,0,sizeof(list));		\
							data.pair		= list;				\
							data.size		= sizeof(list);		\
							data.buf		= buf;				\
							data.bufLen		= sizeof(buf);


static char buf[256];
static ini_pair list[30];
static ini_data data;


static struct key_list
{
	char *key;
	int type;
	void *data;
	int def;
	bool ronly;
} const setting[] = 
{
	{ INI_KEY_FORMAT		,INI_TYPE_STR		, NULL						, false	},//
	{ INI_KEY_DKEY 			,INI_TYPE_BOOL		, &MS_INI_DumpGameKey		, false	},
	
	{ INI_KEY_RID 			,INI_TYPE_BOOL		, &MS_INI_RID				, true	},
	{ INI_KEY_RIDS		 	,INI_TYPE_STR		, NULL						, true	},//
	{ INI_KEY_RIDD 			,INI_TYPE_STR		, NULL						, true	},//
	
	{ INI_KEY_BKUPL 		,INI_TYPE_BOOL		, &MS_INI_RMS				, false	},
	{ INI_KEY_BKUPS 		,INI_TYPE_BOOL		, &MS_INI_WMS				, false	},
	
	{ INI_KEY_MENU 			,INI_TYPE_BOOL		, &MS_INI_Menu				, true	},
	{ INI_KEY_DELAY 		,INI_TYPE_DEC		, &MS_INI_Menu_Delay		, false	},
	{ INI_KEY_MENU_T 		,INI_TYPE_STR		, NULL						, false	},//
	{ INI_KEY_MENU_STH 		,INI_TYPE_BOOL		, &MS_INI_Menu_STH			, false	},
	{ INI_KEY_MENU_PX 		,INI_TYPE_DEC		, &MS_INI_Menu_Pos_X		, true	},
	{ INI_KEY_MENU_PY 		,INI_TYPE_DEC		, &MS_INI_Menu_Pos_Y		, true	},
	
	{ INI_KEY_HOTK_SHOW 	,INI_TYPE_STR		, NULL						, true	},
	{ INI_KEY_HOTK_BACK 	,INI_TYPE_STR		, NULL						, true	},
	{ INI_KEY_HOTK_SEL 		,INI_TYPE_STR		, NULL						, true	},
	{ INI_KEY_HOTK_UP 		,INI_TYPE_STR		, NULL						, true	},
	{ INI_KEY_HOTK_DOWN 	,INI_TYPE_STR		, NULL						, true	},
	{ INI_KEY_HOTK_LEFT 	,INI_TYPE_STR		, NULL						, true	},
	{ INI_KEY_HOTK_RIGHT 	,INI_TYPE_STR		, NULL						, true	},
	{ INI_KEY_PLG_USE		,INI_TYPE_BOOL		, &MS_INI_Plg_Use			, false },
	{ INI_KEY_PLG_LFNC		,INI_TYPE_DEC		, &PTBL_Load.Select			, false },
	{ INI_KEY_PLG_SFNC		,INI_TYPE_DEC		, &PTBL_Save.Select			, false },
};


static int key_list_size = sizeof(setting) / sizeof(struct key_list);

// save_cfg(iniSave callback)
static const char* scb_main(const char *key,void *opt)
{
	const char *val = NULL;
	
	if( !strcasecmp(key,INI_KEY_FORMAT) )
	{
		//"File_Format"
		
		switch( MS_INI_SFormat )
		{
			case SFORMAT_AUTO	:val = MS_STR_AUTO		;break;
			case SFORMAT_RAW	:val = MS_STR_RAW		;break;
			case SFORMAT_NORMAL	:val = MS_STR_NORMAL	;break;
		}
	}
	else if( !strcasecmp(key,INI_KEY_MENU_T) )
	{
		//"Menu_Type"
		
		val = ( MS_INI_Menu_Type == LIBM_VIEW_ALL ? MS_STR_ALL : MS_STR_ACTIVE );
	}
	
	logs("scb_main(%d,%s) = %p\n",opt,key,val);
	
	return val;
}

// lcb_main(iniSplitVal callback)
static void cb_split(const char *val,void *opt)
{
	struct Pad_Data_Table
	{
		char *name;
		unsigned int code;
	} const pad_data[] = 
	{
		{ PAD_KEY_SELECT	,PSP_CTRL_SELECT	},
		{ PAD_KEY_START		,PSP_CTRL_START		},
		{ PAD_KEY_UP		,PSP_CTRL_UP		},
		{ PAD_KEY_RIGHT		,PSP_CTRL_RIGHT		},
		{ PAD_KEY_DOWN		,PSP_CTRL_DOWN		},
		{ PAD_KEY_LEFT		,PSP_CTRL_LEFT		},
		{ PAD_KEY_LT		,PSP_CTRL_LTRIGGER	},
		{ PAD_KEY_RT		,PSP_CTRL_RTRIGGER	},
		{ PAD_KEY_TRIANGLE	,PSP_CTRL_TRIANGLE	},
		{ PAD_KEY_CIRCLE	,PSP_CTRL_CIRCLE	},
		{ PAD_KEY_CROSS		,PSP_CTRL_CROSS		},
		{ PAD_KEY_SQUARE	,PSP_CTRL_SQUARE	},
		{ PAD_KEY_HOME		,PSP_CTRL_HOME		},
		{ PAD_KEY_HOLD		,PSP_CTRL_HOLD		},
		{ PAD_KEY_NOTE		,PSP_CTRL_NOTE		},
		{ PAD_KEY_SCREEN	,PSP_CTRL_SCREEN	},
		{ PAD_KEY_VUP		,PSP_CTRL_VOLUP		},
		{ PAD_KEY_VDOWN		,PSP_CTRL_VOLDOWN	},
		{ PAD_KEY_WLAN		,PSP_CTRL_WLAN_UP	},
		{ PAD_KEY_REMOTE	,PSP_CTRL_REMOTE	},
		{ PAD_KEY_DISC		,PSP_CTRL_DISC		},
		{ PAD_KEY_MS		,PSP_CTRL_MS		},
	};
	
	int i;
	
	for( i = 0 ; i < (sizeof(pad_data) / sizeof(struct Pad_Data_Table)) ; i++ )
	{
		if( !strcasecmp(val,pad_data[i].name) )
		{
			*(int*)opt |= pad_data[i].code;
			break;
		}
	}
	
	logs("cb_split(%s,%p) = 0x%08X\n",val,opt,*(int*)opt);
}



// load_cfg(iniLoad callback)
static bool lcb_main(const char *key, char *val,void *opt)
{
	bool ret = true;
	
	logs("icb_main(%s,%s,%p)\n",key,val,opt);
	
	if( !strcasecmp(key,INI_KEY_FORMAT) )
	{
		if( !strcasecmp(val,MS_STR_AUTO) )
		{
			MS_INI_SFormat = SFORMAT_AUTO;
		}
		else if( !strcasecmp(val,MS_STR_RAW) )
		{
			MS_INI_SFormat = SFORMAT_RAW;
		}
		else if( !strcasecmp(val,MS_STR_NORMAL) )
		{
			MS_INI_SFormat = SFORMAT_NORMAL;
		}
		else
		{
			ret = false;
		}
	}
	else if( !strcasecmp(key,INI_KEY_RIDS) )
	{
		//RID_SFO"
		
		if( !strcasecmp(val,MS_STR_TYPE1) )
		{
			MS_INI_RID_SFO = RID_SFO1;
		}
		else if( !strcasecmp(val,MS_STR_TYPE2) )
		{
			MS_INI_RID_SFO =RID_SFO2;
		}
		else if( !strcasecmp(val,MS_STR_TYPE3) )
		{
			MS_INI_RID_SFO = RID_SFO3;
		}
		else
		{
			ret = false;
		}
	}
	else if( !strcasecmp(key,INI_KEY_RIDD) )
	{
		//"RID_DEC"
		
		if( !strcasecmp(val,MS_STR_TYPE1) )
		{
			MS_INI_RID_DEC = RID_DEC1;
		}
		else if( !strcasecmp(val,MS_STR_TYPE2) )
		{
			MS_INI_RID_DEC = RID_DEC2;
		}
		else if( !strcasecmp(val,MS_STR_TYPE3) )
		{
			MS_INI_RID_DEC = RID_DEC3;
		}
		else
		{
			ret = false;
		}
	}
	else if( strcasestr(key,"Menu_Key_") )
	{
		u32 pad = 0;
		
		iniSplitVal(val,"+",cb_split,&pad);
		
		//ǂݍ񂾒lɃ`FbN
		if( pad != 0 )
		{
			if( !strcasecmp(key,INI_KEY_HOTK_SHOW) )
			{
				Context.HotKey.Show = pad;
			}
			else if( !strcasecmp(key,INI_KEY_HOTK_BACK) )
			{
				Context.HotKey.Back = pad;
			}
			else if( !strcasecmp(key,INI_KEY_HOTK_SEL) )
			{
				Context.HotKey.Select = pad;
			}
			else if( !strcasecmp(key,INI_KEY_HOTK_UP) )
			{
				Context.HotKey.Up = pad;
			}
			else if( !strcasecmp(key,INI_KEY_HOTK_DOWN) )
			{
				Context.HotKey.Down = pad;
			}
			else if( !strcasecmp(key,INI_KEY_HOTK_LEFT) )
			{
				Context.HotKey.Left = pad;
			}
			else if( !strcasecmp(key,INI_KEY_HOTK_RIGHT) )
			{
				Context.HotKey.Right = pad;
			}
			else
			{
				ret = false;
			}
		}
		else
		{
			ret = false;
		}
	}
	else if( !strcasecmp(key,INI_KEY_MENU_T) )
	{
		if( !strcasecmp(val,MS_STR_ALL) )
		{
			MS_INI_Menu_Type = LIBM_VIEW_ALL;
		}
		else if( !strcasecmp(val,MS_STR_ACTIVE) )
		{
			MS_INI_Menu_Type = LIBM_VIEW_ACTIVE;
		}
		else
		{
			ret = false;
		}
	}
	
	return ret;
}



int load_cfg(const char *file)
{
	int i;
	
	INIT_CFG_DATA();
	
	for( i = 0 ; i < key_list_size ; i++ )
	{
		iniAddKey( &data ,setting[i].key ,setting[i].type ,setting[i].data ,NULL );
	}
	
	logs("Load Config : %s\n", file );
	
	return iniLoad(file ,&data,lcb_main,NULL);
}



void save_cfg(const char *file)
{
	int i;
	void *val;
	
	INIT_CFG_DATA();
	
	for( i = 0 ; i < key_list_size ; i++ )
	{
		if( !setting[i].ronly )
		{
			val = NULL;
			
			if( !strcasecmp(setting[i].key,INI_KEY_PLG_USE) )
			{
				//PLG_USE
				val = &MS_INI_Plg_Use;
			}
			else if( setting[i].type != INI_TYPE_STR )
			{
				val = setting[i].data;
			}
			
			iniAddKey( &data ,setting[i].key ,setting[i].type ,NULL ,val );
		}
	}
	
	iniSave(file ,&data,scb_main,NULL);
}

