#include "common.h"
#include "hook.h"




struct hook_syscall_table_header
{
	struct hook_syscall_table_header *next;
	unsigned int offset;
	unsigned int num;
	unsigned int size;
};







void **hookFindExportAddr( const char *modname, const char *libname, unsigned int nid )
{
	SceModule *module;
	struct SceLibraryEntryTable *entry;
	unsigned int i = 0, *entrytable = NULL;
	
	module = sceKernelFindModuleByName( modname );
	
	if( ! module ) return NULL;
	
	while( i < module->ent_size )
	{
		entry = (struct SceLibraryEntryTable *)( (unsigned int)(module->ent_top) + i );
		
		if( ( libname && ( strcmp( libname, entry->libname ) == 0 ) ) || ( ! libname && ! entry->libname ) )
		{
			entrytable = entry->entrytable;
			break;
		}
		
		i += entry->len * sizeof( uint32_t );
	}
	
	if( ! entrytable || ! entry->stubcount ) return NULL;
	
	for( i = 0; i < entry->stubcount; i++ )
	{
		if( entrytable[i] == nid ) return (void **)&(entrytable[entry->vstubcount + entry->stubcount + i]);
	}
	
	return NULL;
}



void **hookFindSyscallAddr( void **addr )
{
	void **cop0_ctrl_register_12;
	struct hook_syscall_table_header *syscall_table;
	void **syscall_entry;
	unsigned int i, total_entries;
	
	if( ! addr ) return NULL;
	
	asm( "cfc0 %0, $12;" : "=r"( cop0_ctrl_register_12 ) );
	
	syscall_table = *cop0_ctrl_register_12;
	
	syscall_entry = (void **)( (unsigned int)syscall_table + sizeof( struct hook_syscall_table_header ) );
	
	total_entries = ( syscall_table->size - sizeof( struct hook_syscall_table_header ) ) / sizeof( void* );
	
	for( i = 0; i < total_entries; i++ )
	{
		if( syscall_entry[i] == *addr ) return &(syscall_entry[i]);
	}
	
	return NULL;
}



void hookFunc( void **addr, void *hookfunc )
{
	unsigned int intc;
	
	if( ! addr || ! hookfunc ) return;
	
	intc = pspSdkDisableInterrupts();
	
	*addr = hookfunc;
	
	sceKernelDcacheWritebackInvalidateRange( addr, sizeof( addr ) );
	sceKernelIcacheInvalidateRange( addr, sizeof( addr ) );
	
	pspSdkEnableInterrupts( intc );
}

